/*
 * A page collection abstraction providing pages from a queue to users.
 *
 * Copyright (C) 2021, 2022 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include "pages.h"



Pages::Pages(Memory *memory, PageQueue *queue)
: _memory(memory), _queue(queue)
{
}

Pages::~Pages()
{
    _queue->close(_memory);
}

/* Remove the first flexpage in the queue. If its owner exists, remove it from
   the owner (retiring the content). Return the flexpage for reuse. */

Flexpage *Pages::remove()
{
    PageOwner *owner;
    Flexpage *flexpage;

    _queue->pop(&owner, &flexpage);

    if (owner != NULL)
        owner->remove(flexpage);

    return flexpage;
}

/* Reserve 'flexpage' by removing it from this collection. */

bool Pages::reserve(PageOwner *owner, Flexpage *flexpage)
{
    return _queue->remove(owner, flexpage);
}

/* Obtain a new flexpage. Return the flexpage or None if no new flexpage could
   be created. */

Flexpage *Pages::flexpage()
{
    Region *region = _memory->region();

    if (region != NULL)
        return new Flexpage(region);
    else
        return remove();
}

/* Queue an entry associating the given 'owner' and 'flexpage'. */

void Pages::queue(PageOwner *owner, Flexpage *flexpage)
{
    _queue->push(owner, flexpage);
}

/* Push an entry for 'flexpage' without owner association for immediate
   reuse. */

void Pages::release(Flexpage *flexpage)
{
    _queue->push_front(NULL, flexpage);
}

// vim: tabstop=4 expandtab shiftwidth=4
